@@ -1,3 +1,8 @@
+import logging
+
+logger = logging.getLogger(__name__)
+
+
 class InMemoryDataBase(object):
     def __init__(self):
         self.db = {}
@@ -6,11 +11,20 @@ class InMemoryDataBase(object):
         if self.db.get(key):
             return 1
 
+    def __getitem__(self, key):
+        return self.db.get(key, None)
+
+    def __setitem__(self, key, value):
+        self.db[key] = value
+
+    def __delitem__(self, key):
+        del self.db[key]
+
     def set(self, key, value):
         self.db[key] = value
 
-    def get(self, key):
-        return self.db.get(key, None)
+    def get(self, key, default=None):
+        return self.db.get(key, default)
 
     def delete(self, key):
         if self.db.get(key):
