@@ -22,6 +22,7 @@ class Introspection(Endpoint):
     def __init__(self, **kwargs):
         Endpoint.__init__(self, **kwargs)
         self.offset = kwargs.get("offset", 0)
+        self.enable_claims_per_client = kwargs.get("enable_claims_per_client", False)
 
     def get_client_id_from_token(self, endpoint_context, token, request=None):
         """
@@ -35,6 +36,23 @@ class Introspection(Endpoint):
         sinfo = endpoint_context.sdb[token]
         return sinfo["authn_req"]["client_id"]
 
+    def _get_client_claims(self, token):
+        client_id = self.get_client_id_from_token(self.endpoint_context, token)
+        client = self.endpoint_context.cdb.get(client_id, {})
+        return client.get("introspection_claims")
+
+    def _get_user_info(self, token_info):
+        user_id = self.endpoint_context.sdb.sso_db.get_uid_by_sid(token_info["sid"])
+        return self.endpoint_context.userinfo(user_id, client_id=None)
+
+    def _add_claims(self, token_info, claims, payload):
+        user_info = self._get_user_info(token_info)
+        for attr in claims:
+            try:
+                payload[attr] = user_info[attr]
+            except KeyError:
+                pass
+
     def _introspect(self, token):
         try:
             info = self.endpoint_context.sdb[token]
@@ -42,9 +60,7 @@ class Introspection(Endpoint):
             return None
 
         # Make sure that the token is an access_token or a refresh_token
-        if token not in info.get("access_token") and token != info.get(
-            "refresh_token"
-        ):
+        if token != info.get("access_token") and token != info.get("refresh_token"):
             return None
 
         eat = info.get("expires_at")
@@ -89,6 +105,12 @@ class Introspection(Endpoint):
 
         _resp.update(_info)
         _resp.weed()
+
+        if self.enable_claims_per_client:
+            client_claims = self._get_client_claims(_token)
+            if client_claims:
+                self._add_claims(_info, client_claims, _resp)
+
         _resp["active"] = True
 
         return {"response_args": _resp}
