@@ -2,8 +2,10 @@ import shelve
 
 
 class ShelveDataBase(object):
-    def __init__(self, filename, flag='c', protocol=None, writeback=False):
-        self.db = shelve.open(filename=filename, flag=flag, protocol=protocol, writeback=writeback)
+    def __init__(self, filename, flag="c", protocol=None, writeback=False):
+        self.db = shelve.open(
+            filename=filename, flag=flag, protocol=protocol, writeback=writeback
+        )
 
     def __contains__(self, key):
         if key in self.db:
@@ -11,6 +13,15 @@ class ShelveDataBase(object):
         else:
             return False
 
+    def __setitem__(self, key, value):
+        self.db[key] = value
+
+    def __getitem__(self, key):
+        return self.get(key, None)
+
+    def __delitem__(self, key):
+        return self.delete(key)
+
     def set(self, key, value):
         self.db[key] = value
 
@@ -33,4 +44,7 @@ class ShelveDataBase(object):
         self.db.clear()
 
     def close(self):
-        self.db.close()
\ No newline at end of file
+        self.db.close()
+
+    def sync(self):
+        self.db.sync()
